/*
  Algorithm.cpp

  Algorithm Implementation

  Autor: Daniela Slavkova
*/

/*--------------------------------------------------------------------------
  Body Identification
--------------------------------------------------------------------------*/
#define ALGORITHM

/*-------------------------------------------------------------------------
  Included files
--------------------------------------------------------------------------*/
#include "stdafx.h"
#include <IOSTREAM>
#include <FSTREAM>
#include <STRING>
#include <vector>
#include <list>
#include <IOMANIP>
#include <algorithm>
#include <afxdao.h>

#include "Algorithm.h"
#include "Alphabet.h"
#include "Database.h"
#include "ProcessingErrors.h"

/*--------------------------------------------------------------------------
  Local constants
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local macro
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local type
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local data
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Constant local data
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Exported data
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Constant exported data
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local function prototypes
--------------------------------------------------------------------------*/

/*============================================================================
============================== PRIVATE FUNCTIONS =============================
============================================================================*/

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function NewState(): Retrun new state in dictionary  
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
int DictionaryCreation::NewState()
{
	int ResState = NewCurrPos; 

	// create new empty state in Dict
	Array A;
	for(int I = 0; I < CharMapL; I++)
	{
		A.push_back(MissingTransition);
	}
	Dict.push_back(A);

	// initialize the new state in Fin as not final state
	Fin.push_back(0);

	// move index of new states 
	NewCurrPos++;

	// return the new reserved state
	return ResState;
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function Final(int State): Return if state is final
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
int DictionaryCreation::Final(int State)
{
	ASSERT(State < NewCurrPos);
	ASSERT(State > -1);

	return Fin[State];
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function SetFinal(int State, bool Status): Set the status 
  of the state
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void DictionaryCreation::SetFinal(int State, int Status)
{
	ASSERT(State < NewCurrPos);
	ASSERT(State > -1);

	Fin[State] = Status;
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function Transition(int State, char Ch): Return transition 
  in dictionary   
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
int DictionaryCreation::Transition(int State, char Ch)
{
	int Index;
	if(int(Ch) < 0)
	{
		Index = int(Ch) + 256;	
	}
	else
	{
		Index = int(Ch);
	}

	ASSERT(State < NewCurrPos);
	ASSERT(State > -1);
	ASSERT(Index < 256);
	ASSERT(Index > -1);

	if(ASCIICodes[Index] == -1)
	{
		string ErrorMessage = "This symbol is not present in alphabet characters: \'";
		ErrorMessage.append(1, Ch);
		ErrorMessage.append("\'");

		ProcessError(ErrorMessage);

		// mark execution error presentation
		FlagExecutionErrors = true;

		return MissingTransition;
	}
	else
	{
		return Dict[State][ASCIICodes[Index]];		
	}
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function SetTransition(int State1, char Ch, int State2): 
  Set transition from state to state
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void DictionaryCreation::SetTransition(int State1, char Ch, int State2)
{
	int Index;
	if(int(Ch) < 0)
	{
		Index = int(Ch) + 256;	
	}
	else
	{
		Index = int(Ch);
	}
	
	ASSERT(State1 < NewCurrPos);
	ASSERT(State2 < NewCurrPos);

	if(ASCIICodes[Index] == -1)
	{
		string ErrorMessage = "This symbol is not present in alphabet characters: \'";
		ErrorMessage.append(1, Ch);
		ErrorMessage.append("\'");

		ProcessError(ErrorMessage);

		// mark execution error presentation
		FlagExecutionErrors = true;
	}
	else
	{
		Dict[State1][ASCIICodes[Index]] = State2;
	}
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function ClearState(int State): Clear state  
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void DictionaryCreation::ClearState(int State)
{
	ASSERT(State < NewCurrPos);
	
	SetFinal(State, 0);
	
	// delete all transition from State
	for (int I = 0; I < CharMapL; I++)
	{
		SetTransition(State, Characters[I], MissingTransition);
	}
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function CopyState(int State): Copy state in the dictionary  
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
int DictionaryCreation::CopyState(int State)
{
	ASSERT(State < NewCurrPos); 
	
	int NewSt;
	NewSt = NewState();
	SetFinal(NewSt, Final(State));
		
	// copy all transition of State in NewSt 
	for (int I = 0; I < CharMapL; I++)
	{
		SetTransition(NewSt, Characters[I], Transition(State, Characters[I]));
	}
	return NewSt;
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function Member(int State): Return equal state. If such state
  does not exist returns NULL  
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
int DictionaryCreation::Member(int State)
{
	ASSERT(State < NewCurrPos); 
	ASSERT(State > -1); 
	
	// index of the permanent states
	int PermanentState = WordMaxLength + 1;
	 
	// loop trough all permanent states
	while(PermanentState < NewCurrPos)
	{
		// loop trough all characters
		for (int LetterInd = 0; LetterInd < CharMapL; LetterInd++)
		{
			// if there is different transitions, break the loop trough the other characters
			// for this state
			if(Dict[PermanentState][LetterInd] != Dict[State][LetterInd])
			{
				break;
			}
		}
		// whether the states have same transitions and whether the states have same final statuses
		// if it is true then PermanentState is searched state
		if((LetterInd == CharMapL) && (Fin[PermanentState] == Fin[State]))
		{
			return PermanentState;
		}
		PermanentState++;
	}
	return NULL;
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function FindMinimized(int State): Return equal state or new 
  state if there is not equal state 
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
int DictionaryCreation::FindMinimized(int State)
{
	int StateMem;
	StateMem = Member(State);
	// if there is not equal state
	if (StateMem == NULL)
	{
		StateMem = CopyState(State);
	}
	return StateMem;
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function PrepareInputData(char* InFile, char* TableName, 
  char* ColumnName): Find max word size in the dictionary(input file) and sort
  words from the input file in ascending order into SortFile
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
int DictionaryCreation::PrepareInputData(
										  char* InDataBaseFile, 
										  char* TableName,
										  char* ColumnName
										)
{
	// initialize success indicator
	int ReturnCode = 1;

	string Word;

	int TempLength = 0, PrefixLengthPlus1 = 0;

	// Open database and read data
	// name of db
	CDaoDatabase *Database;	
	
	// initialize the database pointer
	Database = NULL;
	
	// close any open database
	CloseDatabase(&Database);
	
	int RetCode = OpenDatabase(&Database, InDataBaseFile, FALSE);
	if (1 == RetCode)
	{
		CDaoRecordset Recordset(Database);

		string Query = "";
		Query.append("SELECT ");
		Query.append(ColumnName);
		Query.append(" FROM ");
		Query.append(TableName);
		
		try
		{
			Recordset.Open(AFX_DAO_USE_DEFAULT_TYPE, Query.c_str());
		}
		catch (CDaoException *e) 
		{
			// create a message to display
			string ErrorMessage = "Couldn't execute database query -> Exception: \n";
			ErrorMessage.append(e->m_pErrorInfo->m_strDescription);
			
			// indicate fatal error
			ReturnCode = 0;

			// not rethrowing, so delete exception
			e->Delete();

			ProcessError(ErrorMessage);

			// mark execution error presentation
			FlagExecutionErrors = true;
		}
		catch (CMemoryException *e)
		{
			// indicate fatal error
			ReturnCode = 0;

			// not rethrowing, so delete exception
			e->Delete();

			ProcessError("Failed to open database -> Memory exception thrown.");
			
			// mark execution error presentation
			FlagExecutionErrors = true;
		}	

		if(ReturnCode)
		{	
			bool FlagAllDataReadable = 1;

			// Move through records and find max word size
			while(!Recordset.IsEOF())
			{
				COleVariant Var;
				
				Var = Recordset.GetFieldValue(0);
				if(Var.vt != VT_BSTR)
				{
					FlagAllDataReadable = 0;
				}
				else
				{
					Word = Var.pcVal;
					TempLength = Word.length();
					if(WordMaxLength < Word.length())
					{
						WordMaxLength = TempLength;
					}
			
					// Convert cyrillic word to lowercase
					Word = LowerCase(Word);

					// fill array with all words from the input file
					ArrayWords.push_back(Word);
				}

				Recordset.MoveNext( );
			}

			// sort the array with the words
			ArrayWords.sort();

			if(FlagAllDataReadable == 0)
			{
				ProcessError("Field value is different from text.");
			}
		}
	}
	else
	{			
		string ErrorMessage = "Couldn't open database file \"";
		ErrorMessage.append(InDataBaseFile);
		ErrorMessage.append("\"");
		
		ReturnCode = 0;

		ProcessError(ErrorMessage);

     	// mark execution error presentation
		FlagExecutionErrors = true;
	}

	// close any open database
	CloseDatabase(&Database);

	return ReturnCode;
}


/*============================================================================
============================= PUBLIC FUNCTIONS ===============================
============================================================================*/

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function CreateMinAutomata(char* InFile): Algorith for 
  creation of  minimal deterministic automata 
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void DictionaryCreation::CreateMinAutomata(char* InFile, char* Table, char* Column)
{
	string CurrentWord, PreviousWord = "";
	int PrefixLengthPlus1 = 0;

	// Preparation of input data for the algorithm
	int ReturnCode = PrepareInputData(InFile, Table, Column);
	
	if(ReturnCode)
	{
		// temp rows in dictionary
		int *TempStates = new int[WordMaxLength + 1];

		// initialize TempStates array
		for(int I = 0; I <= WordMaxLength; I++)
		{
			TempStates[I] = NewState();	
		}

		// create min deterministic automata
		while(!(ArrayWords.empty()))
		{

			CurrentWord = ArrayWords.front();
			ArrayWords.pop_front();
			
			// calculate the length of the longest common prefix of CurrentWord and PreviousWord
			I = 1;
			while((I <= CurrentWord.length()) && (I <= PreviousWord.length()) && (PreviousWord[I-1] == CurrentWord[I-1]))
			{
				I++;
			}
			PrefixLengthPlus1 = I;
			
			// minimize the temp states of the sufix in PreviosWord
			for(I = PreviousWord.length(); I >= PrefixLengthPlus1; I--)
			{
				SetTransition(TempStates[I-1], PreviousWord[I-1], FindMinimized(TempStates[I]));	
			}

			// create the chain of new states for CurrentWord
			for(I = PrefixLengthPlus1; I <= CurrentWord.length(); I++)
			{
				ClearState(TempStates[I]);
				SetTransition(TempStates[I-1], CurrentWord[I-1], TempStates[I]);
			}

			// make the last state for the word final
			SetFinal(TempStates[CurrentWord.length()], 1);

			PreviousWord = CurrentWord;
		} 
		
		// minimize the states of last word
		for(I = CurrentWord.length(); I >= 1; I--)
		{
			SetTransition(TempStates[I-1], CurrentWord[I-1], FindMinimized(TempStates[I]));  
		}
		
		// find initial state of dictionary
		InitialState = FindMinimized(TempStates[0]);
		
		delete [] TempStates;
	}
	else
	{
		string ErrorMessage = "There are problem with the input data";
		
		ProcessError(ErrorMessage);

		// mark execution error presentation
		FlagExecutionErrors = true;
	}
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function SerializeArray()
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void DictionaryCreation::SerializeArray()
{
	fstream FsOut = fstream(SerializeFile, ios::out | ios::trunc);
	
	if(FsOut.is_open())
	{
		// marker
		FsOut << "Automata Serialization File\n";

		// memorize states count
		FsOut << "States count: " << NewCurrPos - (WordMaxLength + 1) << "\n";
		// memorize initial state
		FsOut << "Initial state: " << InitialState - (WordMaxLength + 1) << "\n";
		// memorize alphabet symbols count
		FsOut << "Characters: " << CharMapL << "\n";

		// memorize final state
		FsOut << "Final states: ";
		for(int I = WordMaxLength + 1; I < NewCurrPos; I++)
		{
			if(Fin[I] == 1)
			{
				FsOut << I - (WordMaxLength + 1) << " ";
			}
		}

		FsOut << "\n-------------\n";
		
		for(I = WordMaxLength + 1; I < NewCurrPos; I++)
		{
			for(int J = 0; J < CharMapL; J++)
			{
				// if Dict[I][J] == MissingTransition then just print MissingTransition value 
				if(Dict[I][J] != MissingTransition)
				{
					FsOut << Dict[I][J]  - (WordMaxLength + 1) << " ";
				}	
				else
				{
					FsOut << MissingTransition << " ";
				}
			}

			FsOut << "\n";
		}
		
		// new initialization of class data
		NewCurrPos = NewCurrPos - (WordMaxLength + 1);
		InitialState = InitialState - (WordMaxLength + 1);
		WordMaxLength = MissingTransition;

		FsOut.close();
	}
	else
	{
		string ErrorMessage = "Couldn't open output file to serialization: \"";
		ErrorMessage.append(SerializeFile);
		ErrorMessage.append("\"");

		ProcessError(ErrorMessage);

		// mark execution error presentation
		FlagExecutionErrors = true;
	}
}

/*============================================================================
============================== LOCAL FUNCTIONS ===============================
============================================================================*/


/*============================================================================
============================= EXPORTED FUNCTIONS =============================
============================================================================*/
