/*
  DataBase.cpp

  DataBase Implementation

  Autor: Daniela Slavkova
*/

/*--------------------------------------------------------------------------
  Body Identification
--------------------------------------------------------------------------*/

#define DB

/*-------------------------------------------------------------------------
  Included files
--------------------------------------------------------------------------*/
#include "stdafx.h"
#include <afxdao.h>
#include "Database.h"
#include "ProcessingErrors.h"

/*--------------------------------------------------------------------------
  Local constants
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local macro
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local type
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local data
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Constant local data
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Exported data
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Constant exported data
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local function prototypes
--------------------------------------------------------------------------*/

/*============================================================================
============================= PRIVATE FUNCTIONS ==============================
============================================================================*/

/*============================================================================
============================== PUBLIC FUNCTIONS ========================
============================================================================*/

/*============================================================================
============================== LOCAL FUNCTIONS ===============================
============================================================================*/

/*============================================================================
============================= EXPORTED FUNCTIONS =============================
============================================================================*/

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function OpenDatabase(CDaoDatabase **ppDatabase, CString fileName,
										BOOL bReportNoOpen):
  Open the database from a file (close if necessary). 
  ppDatabase(IN/OUT) is pointer to pointer to database object.
  fileName(IN) is name of .mdb file to open.
  bReportNoOpen(IN) is TRUE by default - if true, if open fails
  because the specified MDB file doesn't exist, report that
  fact to the user.  
  Note: Other errors are reported always.
  Return 1 if success, 0 if non-fatal failure, -1 if fatal failure.
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
int OpenDatabase(CDaoDatabase **ppDatabase, CString fileName, BOOL bReportNoOpen /* = TRUE */)
{
//	AFX_MANAGE_STATE(AfxGetStaticModuleState( ));
	
	// initialize success indicator
	int nReturnCode = 1;

	// close and delete if necessary
	if (*ppDatabase != NULL)
	{
		if ((*ppDatabase)->IsOpen())
			CloseDatabase(ppDatabase);
		delete *ppDatabase;
	}

	// construct new database
	*ppDatabase = new CDaoDatabase;

	// failed to allocate
	if (ppDatabase == NULL)
		return -1; // fatal error

	// now open the database object with error checking
	try
	{
		// When opening a database created with Access 2000 through 
		// MFC DAO classes in Visual C++, you get the following error message: 
		// Unrecognized database format. 
		// This error message occurs because the MFC DAO classes that ship 
		// with Visual C++ 6.0 load DAO 3.5 (Dao350.dll) by default. DAO 3.5 uses Jet 3.5, 
		// which can only open Jet 3.5 format (or earlier) databases. 
		// Access 2000 creates Jet 4.0 format database files, which are unrecognizable to Jet 3.5. 
		// To successfully open an Access 2000 database using the MFC DAO classes, you need to use 
		// DAO 3.6 (Dao360.dll). DAO 3.6 uses Jet 4.0, which can open any available Access database format. 
		// For your application to use version 3.6 of DAO, you must update the version of MFC at run time
		// to MFC version 6.01. To do this depends on whether you are building the application 
		// to use the MFC DLL or to build with the static libraries for MFC.
		// If you are linking with the MFC DLL, you can specify that you want MFC
		// to use DAO 3.6 by inserting the following line of code before you open an Access 2000 database:
		AfxGetModuleState()->m_dwVersion = 0x0601;
		(*ppDatabase)->Open(fileName);
	}
	catch (CDaoException *e)
	{
		// special case->couldn't find the file, so it may be because
		// user specified a new file to open
		if (e->m_pErrorInfo->m_lErrorCode == 3024)
		{
			if (bReportNoOpen)
			{
				// create a message to display
				string ErrorMessage = "Couldn't open database -> Exception: \n";
				ErrorMessage.append(e->m_pErrorInfo->m_strDescription);
				
				ProcessError(ErrorMessage);
			}

			// indicate failure but not fatal
			nReturnCode = 0;
		}
		else // other type of DAO exception->always report
		{
			// create a message to display
			string ErrorMessage = "Couldn't open database -> Exception: \n";
			ErrorMessage.append(e->m_pErrorInfo->m_strDescription);

			// indicate fatal error
			nReturnCode = -1;

			ProcessError(ErrorMessage);
		}

		// not rethrowing, so delete exception
		e->Delete();

		delete *ppDatabase;
		*ppDatabase = NULL;
	}
	catch (CMemoryException *e)
	{
		// output status
		ProcessError("Failed to open database -> Memory exception thrown.");

		// not rethrowing, so delete exception
		e->Delete();

		delete *ppDatabase;
		*ppDatabase = NULL;

		// indicate fatal error
		nReturnCode = -1;
	}

	return nReturnCode;
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function CloseDatabase(CDaoDatabase **ppDatabase):
  Close the database and destruct it.
  ppDatabase(IN/OUT) is pointer to pointer to database to close and
  destruct.
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void CloseDatabase(CDaoDatabase **ppDatabase)
{
	// only process if the database object exists
	if (*ppDatabase != NULL)
	{
		if ((*ppDatabase)->IsOpen())
		{
			(*ppDatabase)->Close();
		}
		// closing doesn't delete the object
		delete *ppDatabase;
		*ppDatabase = NULL;
	}
	AfxDaoTerm();
}