/*
  Levenstien.cpp

  Levenstien Implementation

  Autor: Daniela Slavkova
*/

/*--------------------------------------------------------------------------
  Body Identification
--------------------------------------------------------------------------*/
#define LEVENSTIEN

/*-------------------------------------------------------------------------
  Included files
--------------------------------------------------------------------------*/
#include "stdafx.h"
#include <IOSTREAM>
#include <FSTREAM>
#include <STRING>

#include "Levenstien.h"

/*--------------------------------------------------------------------------
  Local constants
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local macro
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local type
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local data
--------------------------------------------------------------------------*/
//int LevenshtienDistance[N][M];

/*--------------------------------------------------------------------------
  Constant local data
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Exported data
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Constant exported data
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local function prototypes
--------------------------------------------------------------------------*/


/*============================================================================
============================= PRIVATE FUNCTIONS ==============================
============================================================================*/
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function Minimu(int a, int b): Minimum of a and b
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
int Levenstien::Minimum(int a, int b)
{
	if(a < b)
	{
		return a;
	}
	else
	{
		return b;
	}
}

/*============================================================================
============================== PUBLIC FUNCTIONS ==============================
============================================================================*/
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function LevenshtienDistanceCalc(string T, string P): 
  Calculate Levenstien Distance between two strings - text T and pattern P
  
  Levenshtien Distance Algorithm 
  Algorithm - We start by defining the boundary cases and then iteratively 
  build up values until we have LevenshtienDistance[LengthT][LengthP]: 
  L(e, P) = |P|
  L(T, e) = |T|
              / L(T, P) if a = b 
  L(Ta, Pb) =  
              \ 1 + min{L(T, P), L(Ta, P), L(T, Pb)} if not a = b
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
int Levenstien::LevenstienDistanceCalc(string T)
{
	int i, j;
	
	int LengthT = T.length();		// length of Text	

	// boundary conditions 
	for(j = 0; j <= LengthT; j++)
	{
		LevenshtienDistanceTable[0][j] = j;
	}
	
	for(i = 0; i <= PatternLength; i++)
	{
		LevenshtienDistanceTable[i][0] = i;
	}
	
	// filling of the matrix LevenshtienDistance[][]
	for(i = 1; i <= PatternLength; i++)
	{
		for(j = 1; j <= LengthT; j++)
		{
			char a = T.at(j-1);	
			char b = Pattern.at(i-1);
					
			// If there is a match between the last character of T and P
			if(a == b)
			{
				LevenshtienDistanceTable[i][j] = LevenshtienDistanceTable[i-1][j-1];
			}
			// If there is a match between the last character of T and P
			else
			{
				LevenshtienDistanceTable[i][j] = 1 + Minimum(
																Minimum(
																			LevenshtienDistanceTable[i-1][j], 
																			LevenshtienDistanceTable[i][j-1]
																		), 
																LevenshtienDistanceTable[i-1][j-1]
															);
			}
		}
	}

	// result
	return LevenshtienDistanceTable[PatternLength][LengthT];
}
  
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of void InsertIntoTable(char Ch, int Column): Insert Column in
  LevenstienDistanceTable
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
bool Levenstien::InsertIntoTable(char Ch, int Column)
{	
//	// filling of the matrix LevenshtienDistance[][]
//	for(int i = 1; i <= PatternLength; i++)
//	{
//		char b = Pattern.at(i-1);
//					
//		// If there is a match between the last character of T and P
//		if(Ch == b)
//		{
//			LevenshtienDistanceTable[i][Column] = LevenshtienDistanceTable[i-1][Column-1];
//		}
//		// If there is a match between the last character of T and P
//		else
//		{
//			LevenshtienDistanceTable[i][Column] = 1 + Minimum(
//																Minimum(
//																			LevenshtienDistanceTable[i-1][Column], 
//																			LevenshtienDistanceTable[i][Column-1]
//																		), 
//																LevenshtienDistanceTable[i-1][Column-1]
//															 );
//		}
//	}
	
	bool FlagValue = false;

	ASSERT(IndexFirstColumnArray > -1);
	ASSERT(FirstColumns[IndexFirstColumnArray] > -1) ;
	ASSERT(FirstColumns[IndexFirstColumnArray] < 256) ;

	ASSERT(Column > -1);
	ASSERT(Column < 256);


	if(Column == FirstColumns[IndexFirstColumnArray])
	{
		InitializeInitial(Column);
		FlagValue = true;
	}
	else
	{
		// filling of the matrix LevenshtienDistance[][]
		for(int i = (Column-FirstColumns[IndexFirstColumnArray]) - GoodLevenstienDist; i <= (Column-FirstColumns[IndexFirstColumnArray]) + GoodLevenstienDist; i++)
		{
			if((i <= 0) || (i > PatternLength))
			{
			
			}
			else
			{
				char b = Pattern.at(i-1);
							
				// If there is a match between the last character of T and P
				if(Ch == b)
				{
					LevenshtienDistanceTable[i][Column] = LevenshtienDistanceTable[i-1][Column-1];
				}
				// If there is a match between the last character of T and P
				else
				{
					if(i == (Column-FirstColumns[IndexFirstColumnArray]) - GoodLevenstienDist)
					{
						LevenshtienDistanceTable[i][Column] = 1 + Minimum(
																			LevenshtienDistanceTable[i][Column-1],
																			LevenshtienDistanceTable[i-1][Column-1]
																		 );
					}
					else
					{
						if(i == (Column-FirstColumns[IndexFirstColumnArray]) + GoodLevenstienDist)
						{
							LevenshtienDistanceTable[i][Column] = 1 + Minimum(
																				LevenshtienDistanceTable[i-1][Column], 		
																				LevenshtienDistanceTable[i-1][Column-1]
																			 );
								
						}
						else
						{
							LevenshtienDistanceTable[i][Column] = 1 + Minimum(
																				Minimum(
																							LevenshtienDistanceTable[i-1][Column], 
																							LevenshtienDistanceTable[i][Column-1]
																					   ), 
																				LevenshtienDistanceTable[i-1][Column-1]
																			 );
						}
					}
				}
				
				if(LevenshtienDistanceTable[i][Column] < GoodLevenstienDist + 1)
				{
					FlagValue = true;
				}
			}
		}
	}
				
	return FlagValue;
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of int GetLevenstienDist(int Index): Return Levenstien distance
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
int Levenstien::GetLevenstienDist(int Index)
{
//	ASSERT(Index > -1);
//	// result
//	return LevenshtienDistanceTable[PatternLength][Index];

	ASSERT(Index > -1);
	ASSERT(Index < 256);

	ASSERT(FirstColumns[IndexFirstColumnArray] > -1);
	ASSERT(FirstColumns[IndexFirstColumnArray] < 256);
	
	ASSERT(IndexFirstColumnArray > -1);
	ASSERT(IndexFirstColumnArray < 256);
	
	if((PatternLength >= (Index-FirstColumns[IndexFirstColumnArray]) - GoodLevenstienDist) && (PatternLength <= (Index-FirstColumns[IndexFirstColumnArray]) + GoodLevenstienDist))
	{
		// result
		return LevenshtienDistanceTable[PatternLength][Index];
	}
	else
	{
		return -1;
	}
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of void InitializeInitial(int Column): Initialize initial
  row and column
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void Levenstien::InitializeInitial(int Column)
{
	// boundary conditions 
	for(int j = Column; j <= M; j++)
	{
		LevenshtienDistanceTable[0][j] = j-Column;
	}

	for(int i = 0; i <= PatternLength; i++)
	{
		LevenshtienDistanceTable[i][Column] = i;	
	}
}

/*============================================================================
============================== LOCAL FUNCTIONS ===============================
============================================================================*/

/*============================================================================
============================= EXPORTED FUNCTIONS =============================
============================================================================*/
