/*
  RequestDictionary.cpp

  Request Dictionary Implementation

  Autor: Daniela Slavkova
*/

/*--------------------------------------------------------------------------
  Body Identification
--------------------------------------------------------------------------*/
#define REQUESTDICTIONARY

/*-------------------------------------------------------------------------
  Included files
--------------------------------------------------------------------------*/
#include "stdafx.h"
#include <IOSTREAM>
#include <FSTREAM>
#include <STRING>
#include <list>
#include <IOMANIP>
#include <algorithm>

#include "RequestDictionary.h"
#include "Alphabet.h"
#include "Levenstien.h"
#include "ProcessingErrors.h"


/*--------------------------------------------------------------------------
  Local constants
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local macro
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local type
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local data
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Constant local data
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Exported data
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Constant exported data
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
  Local function prototypes
--------------------------------------------------------------------------*/

/*============================================================================
=============== PRIVATE FUNCTIONS OF DESERIALIZEDDICTIONARY ==================
============================================================================*/

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function InitializeDictionary(): Initialize empty Dictionary
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void Dictionary::InitializeDictionary()
{
	// initialize new empty Dict and		
	// initialize final states as non final
	if(NewCurrPos > -1)
	{
		Dict = new int[NewCurrPos][CharMapL];
		Fin = new int[NewCurrPos];
	
		for(int K = 0; K < NewCurrPos; K++)
		{
			for(int L = 0; L < CharMapL; L++)
			{
				Dict[K][L] = MissingTransition;
			}

			Fin[K] = 0;
		}
	}
	else
	{
		string ErrorMessage = "Missing automata: -1 count automata states";

		ProcessError(ErrorMessage);	

		// mark execution error presentation
		FlagExecutionErrors = true;
	}

}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function Traversing(int State): Traverse State
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void Dictionary::Traversing(int State)
{	
	ASSERT (State > -1);
	
	if(Fin[State])
	{
		cout << "\n";
		PrintVisited();
	}
	
	for(int I = 0; I < CharMapL; I++)
	{
		if(Dict[State][I] != MissingTransition)
		{
			Visited.push_back(Characters[I]);

			Traversing(Dict[State][I]);

			Visited.pop_back();
		}
	}
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function PrintVisited(): Print visited word
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void Dictionary::PrintVisited()
{
	Word::iterator I;
	
	for (I = Visited.begin(); I != Visited.end(); ++I)
        cout << *I;
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function ToString(): Convert visited word to string
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
string Dictionary::ToString(int MinFindLevenshtienDist)
{
	Word::iterator I;
	string ResStr = "";
	char Num[2];
	
	// convert number to char*
	_itoa(MinFindLevenshtienDist, Num, 10);
	
	// append levenshtien dist to result
	ResStr.append(Num);
	ResStr.append(" ");

	// print result
	for (I = Visited.begin(); I != Visited.end(); ++I)
	{
		ResStr.append(1, *I);
	}

	return ResStr;
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function RequestAnswer(): Request answer
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
int Dictionary::RequestAnswer()
{
//	bool Res = true;
//	int I = 0;
//	int Len = LevenstienObj.size();
//
//	int MinWordLevenshtein = 257, WordLevenshtien;
//
//	// while there are LevenstienObj
//	// check if they satisfy given Levenstien distance
//	while((I < Len) && (Res != false))
//	{	
//		if(MarkersDisplay[I] != -1)
//		{
//			WordLevenshtien = FindedDist[I];
//			if(WordLevenshtien < MinWordLevenshtein)
//			{
//				MinWordLevenshtein = WordLevenshtien; 
//			}
//		}
//		else
//		{
//			MinWordLevenshtein = -1;
//			Res = false;
//		}
//
//		I++;
//	}

	bool Res = true;
	int I = 0;
	int Len = LevenstienObj.size();
	int MinWordLevenshtein = 257;

	// while there are LevenstienObj
	// check if they satisfy given Levenstien distance
	while((I < Len) && (Res != false))
	{	
		if(!MarkersDisplay[I].empty())
		{
			
		}
		else
		{
			MinWordLevenshtein = -1;
			Res = false;
		}

		I++;
	}

	if(Res == false)
	{
		
	}
	else
	{
		for(I = 0; I < Len; I++)
		{
			for(int J = 0; J < FindedDist[I].size(); J++)
			{
				if(FindedDist[I][J] < MinWordLevenshtein)
				{
					MinWordLevenshtein = FindedDist[I][J];	
				}
			}
		}
	}
	
	return MinWordLevenshtein;
}


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function Requesting(string T, int State): Request for string T
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void Dictionary::Requesting(int State, int Index,  int PositionString)
{	
	ASSERT (State > -1);
	ASSERT (Index > -1);
	ASSERT (PositionString < LevenstienObj.size());
	
	// if State is final
	// then try to output answer
	if(Fin[State])
	{		
		if(PositionString == LevenstienObj.size() - 1)
		{
			int Distance = LevenstienObj[PositionString]->GetLevenstienDist(Index - 1);

			if((Distance != -1) && (Distance <= LevenstienObj[PositionString]->GoodLevenstienDist))
			{
				MarkersDisplay[PositionString].push_back(Index - 1);
				FindedDist[PositionString].push_back(Distance);				
			}	
		}

		// if for all requested words exist approximation words in the given sequence
		int MinLevenshtien = RequestAnswer();
		if(MinLevenshtien != -1)
		{
			SearchedWords.push_back(ToString(MinLevenshtien));
		}
	}

	// unmark current marker 
	if(MarkersDisplay[PositionString].back() == Index - 1)
	{
		MarkersDisplay[PositionString].pop_back();
		FindedDist[PositionString].pop_back();
	}	

	// for all transitions of State
	for(int I = 0; I < CharMapL; I++)
	{
		if(Dict[State][I] != MissingTransition)
		{
			// read letter
			Visited.push_back(Characters[I]);

			// if readed letter is ' '
			// do special initialization or search for next word from requested sequence
			if(Characters[I] == ' ')
			{							
				int Distance = LevenstienObj[PositionString]->GetLevenstienDist(Index - 1);
					
				if((Distance != -1) && (Distance <= LevenstienObj[PositionString]->GoodLevenstienDist))
				{	
					MarkersDisplay[PositionString].push_back(Index - 1);
					FindedDist[PositionString].push_back(Distance);				
				}	
				
				// if current searched word satisfies the Levenstien distance
				// search in the rest of the dictionary for next searched word from the given sequence
				if((PositionString < LevenstienObj.size() - 1) && (Distance != -1) && (Distance <= LevenstienObj[PositionString]->GoodLevenstienDist))
				{
					Requesting(Dict[State][I], 1, PositionString + 1);
				}
				// move calculation of Levenstien distance to the index after ' '
				else
				{					
					LevenstienObj[PositionString]->IndexFirstColumnArray++;
					LevenstienObj[PositionString]->FirstColumns[LevenstienObj[PositionString]->IndexFirstColumnArray] = Index;
		
					LevenstienObj[PositionString]->InsertIntoTable(Characters[I], Index);
					Requesting(Dict[State][I], Index + 1, PositionString);							
				}

			}
			// continue recursion
			else
			{
				LevenstienObj[PositionString]->InsertIntoTable(Characters[I], Index);
				Requesting(Dict[State][I], Index + 1, PositionString);	
			}

			char Ch = Visited.back();
			Visited.pop_back();

			if(Ch == ' ')
			{		
				if(MarkersDisplay[PositionString].back() == Index - 1)
				{
					MarkersDisplay[PositionString].pop_back();
					FindedDist[PositionString].pop_back();
				}	
				
				int Distance = LevenstienObj[PositionString]->GetLevenstienDist(Index - 1);
				if((PositionString < LevenstienObj.size() - 1) && (Distance != -1) && (Distance <= LevenstienObj[PositionString]->GoodLevenstienDist))
				{
				}
				// move calculation of Levenstien distance to the index before ' '
				else
				{					
					LevenstienObj[PositionString]->IndexFirstColumnArray--;	
					LevenstienObj[PositionString]->InitializeInitial(LevenstienObj[PositionString]->FirstColumns[LevenstienObj[PositionString]->IndexFirstColumnArray]);
				}
			}
		}
	}
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function PrintGoodWords(char* ResFile): Request for string T
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void Dictionary::PrintGoodWords(char* ResFile)
{
	fstream FsOut = fstream(ResFile, ios::out | ios::trunc);

	SearchedWords.sort();

	if(FsOut.is_open())
	{
		// while there are good results
		// print them
		while(!SearchedWords.empty())
		{
			FsOut << UpperCase(SearchedWords.front()) << "\n";
			SearchedWords.pop_front();
		}
		FsOut.close();
	}
	else
	{
		string ErrorMessage = "Couldn't open result file \"";
		ErrorMessage.append(ResFile);
		ErrorMessage.append("\"");

		ProcessError(ErrorMessage);
     
		// mark execution error presentation
		FlagExecutionErrors = true;
	}
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function TranslateWord(string T): Translate string T to the
  given alphabet
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
string Dictionary::TranslateWord(string T)
{
	string ResStr = "";
	int Index;
	
	// for all letters of T
	for(int I = 0; I < T.length(); I++)
	{
		if((int)T[I] < 0)
		{
			Index = (int)T[I] + 256;	
		}
		else
		{
			Index = (int)T[I];
		}
	
		// assert if Ch is not ASCII code
		ASSERT(Index < 256);
		ASSERT(Index > -1);

		// if given symol is not present in the alphabet
		if(ASCIICodes[Index] == -1)
		{
			string ErrorMessage = "This symbol is not present in alphabet characters: \'";
			ErrorMessage.append(1, T[I]);
			ErrorMessage.append("\'");

			ProcessError(ErrorMessage);

//			// mark execution error presentation
//			FlagExecutionErrors = true;
		}
		// translate word
		else
		{
			ResStr.append(1, Characters[ASCIICodes[Index]]);
		}
	}

	return ResStr;
}

/*============================================================================
=============== PUBLIC FUNCTIONS OF DESERIALIZEDDICTIONARY ===================
============================================================================*/
  
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function DeserializeArray(): Deserialize Dictionary from
  saved file
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void Dictionary::DeserializeArray()
{
	fstream FsIn = fstream(SerializeFile, ios::in);

	string Line;
	int Pos = 0, StatesPos, InitialStatePos, CharNumbersPos, FinalStatesPos;
	int BegPos = 0;
	int I = 0, Value, FinStatesLine;
	string IntValue, StatesValue, InitialStateValue, CharNumbersValue;
	
	// deserialized file is open correctly
	if(FsIn.is_open())
	{
		// marker
		getline(FsIn, Line);
		// opened file is Dictionary file
		if(!Line.compare("Automata Serialization File"))
		{	
			// find number of states
			getline(FsIn, Line);
			StatesPos = Line.find(':', 0);
			StatesValue = Line.substr(StatesPos + 2, Line.length());
			Value = atoi(StatesValue.c_str());
			// states count initialization (NewCurrPos data in class Dictionary)
			NewCurrPos = Value;

			// find initial state index
			getline(FsIn, Line);
			InitialStatePos = Line.find(':', 0);
			InitialStateValue = Line.substr(InitialStatePos + 2, Line.length());
			Value = atoi(InitialStateValue.c_str());
			// initial state initialization (InitialState data in class Dictionary)
			InitialState = Value;

			// find number of characters in alphabet
			getline(FsIn, Line);
			CharNumbersPos = Line.find(':', 0);
			CharNumbersValue = Line.substr(CharNumbersPos + 2, Line.length());
			Value = atoi(CharNumbersValue.c_str());

			// initialize new empty Dict
			// and initialize final states as non final
			InitializeDictionary();

			// find final states
			getline(FsIn, Line);
			FinalStatesPos = Line.find(':', 0);		
			FinalStatesPos = FinalStatesPos + 2;
			FinStatesLine = Line.length();
			while(FinalStatesPos != FinStatesLine)
			{
				Pos = Line.find(' ', FinalStatesPos);
				IntValue = Line.substr(FinalStatesPos, Pos - FinalStatesPos);	
				// Value is index of final state
				Value = atoi(IntValue.c_str());
				Fin[Value] = 1;

				FinalStatesPos = Pos + 1;
			}
			
			// slip line "-----------"
			getline(FsIn, Line);
					
			// complete Dict with all transition in the min deterministic automata
			I = 0;
			while(!(FsIn.eof()))
			{			
				getline(FsIn, Line);

				// skip last new line in serialized file
				if(Line == "")
				{
					break;
				}
		
				for(int J = 0; J < CharMapL; J++)
				{
					Pos = Line.find(' ', BegPos);
					IntValue = Line.substr(BegPos, Pos - BegPos);
					// Value is transition in automata from state with index I by means of character with index J
					Value = atoi(IntValue.c_str());
					Dict[I][J] = Value;

					BegPos = Pos + 1;
				}

				I++;
				BegPos = 0;

				FlagDictDeserialization = true;
			}
		}
		// opened file is not Dictionary automata file
		else
		{
			string ErrorMessage = "Couldn't deserialize file \"";
			ErrorMessage.append(SerializeFile);
			ErrorMessage.append("\". It is not serialized with AnlysisServer module");

			ProcessError(ErrorMessage);

			// mark execution error presentation
			FlagExecutionErrors = true;
		}
		
		FsIn.close();
	}
	// deserialized file could not be open
	else
	{
		string ErrorMessage = "Couldn't open serialized file \"";
		ErrorMessage.append(SerializeFile);
		ErrorMessage.append("\"");

		ProcessError(ErrorMessage);

		// mark execution error presentation
		FlagExecutionErrors = true;
	}
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function TraversingDictionary(): Traverse Dictionary
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void Dictionary::TraversingDictionary()
{
	Traversing(InitialState);
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function RequestDictionary(string T): Request Dictionary
  for string T
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void Dictionary::RequestDictionary(string Request, int LevenstienDistance)
{	
	// if serialization is finished correctly
	if(FlagDictDeserialization == true)
	{
		string TranslatedWord = TranslateWord(LowerCase(Request));
		TranslatedWord = RemoveLastIntervals(TranslatedWord);
		
		// search for prcise word
		if(LevenstienDistance == 0)
		{
			if(FindRequest(TranslatedWord) == TranslatedWord)
			{
				SearchedWords.push_back(TranslatedWord);			
			}

			return;
		}
		
		string Word = "";
		
		for(int I = 0; I < TranslatedWord.length(); I++)
		{
			// find word from requested sequence
			if(TranslatedWord[I] == ' ')
			{
				// cteate new Levenstien object for every word from sequence
				Word = OperateAbbreviationInRequest(Word);
				if(!Word.empty())
				{
//					LevenstienObj.push_back(new Levenstien(Word, LevenstienDistance));
//					VecInt VecDisp;
//					MarkersDisplay.push_back(VecDisp);
//					VecInt VecFind;
//					FindedDist.push_back(VecFind);
					NewLevenshtienObject(Word,LevenstienDistance);
					Word = "";
				}
			}
			else
			{
				// remove requested abbreviations
				if(TranslatedWord[I] == '.')
				{
					Word = "";	
				}
				else
				{
					Word.append(1, TranslatedWord[I]);
				}
			}
		}

		// add last word even it is empty
		Word = OperateAbbreviationInRequest(Word);
//		LevenstienObj.push_back(new Levenstien(Word, LevenstienDistance));
//		VecInt VecDisp;
//		MarkersDisplay.push_back(VecDisp);
//		VecInt VecFind;
//		FindedDist.push_back(VecFind);
		NewLevenshtienObject(Word,LevenstienDistance);

		// make request - begin recursion
		Requesting(InitialState, 1, 0);
		
		// free memory 
		while(!LevenstienObj.empty())
		{
			LevenstienObj.pop_back();
			MarkersDisplay.pop_back();
			FindedDist.pop_back();	
		}
	}
	// if there is rpoblem with deserialization
	else
	{
		ProcessError("Couldn't deserialize dictionary");	

    	// mark execution error presentation
		FlagExecutionErrors = true;
	}
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function FindRequest(string Request): 
  Find requested word if it exists  
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
string Dictionary::FindRequest(string Request)
{	
	int I = 0, Index;
	int State = InitialState;
	char Ch;
	string Res = "";

	// read word from dictionary if it exists
	while(I < Request.length())
	{
		Ch = Request[I];
		if(Ch < 0)
		{
			Index = (int)Ch + 256;	
		}
		else
		{
			Index = (int)Ch;
		}
		
		ASSERT(State > -1);
		ASSERT(State > ASCIICodes[Index]);
		ASSERT(Index > -1);
		
		// if there is such transition
		// move to next letter
		if(Dict[State][ASCIICodes[Index]] != MissingTransition)
		{
			Res.append(1, Ch);
			State = Dict[State][ASCIICodes[Index]];
		}
		// if there is not transition
		// return empty word to indicate unsuccess
		else
		{
			return "";
		}

		I++;
	}

	// if readed word is present in dictionary
	// return success
	if(Fin[State])
	{
		return Res;
	}
	// if readed word is not present in dictionary
	// return unsuccess
	else
	{
		return "";
	}
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function OperateAbbreviationInRequest(string Request): 
  Remove dots from request and operates abbreviation 
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
string Dictionary::OperateAbbreviationInRequest(string Request)
{	
	// if there are whole words in request, that have abbreviations in database 
	for(int I = 0; I < AbbreviationNumber; I++)
	{
		// replace whole word with abbreviation => replace whole word with ""
		// to skip abbreviation
		if(Request == AbbreviationMap[I][0])
		{
			return ""; 
		}
	}

	return Request;
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function RemoveLastIntervals(string Request): 
  Remove last intervals in request
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
string Dictionary::RemoveLastIntervals(string Request)
{
	int I = Request.size() - 1;
	
	while(Request[I] == ' ')
	{
		Request = Request.substr(0, I);
		I--;
	}

	return Request;
}

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
  Description of function RemoveLastIntervals(string Request): 
  Remove last intervals in request
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void Dictionary::NewLevenshtienObject(string Pattern, int LevenshtienDist)
{
	LevenstienObj.push_back(new Levenstien(Pattern, LevenshtienDist));
	VecInt VecDisp;
	MarkersDisplay.push_back(VecDisp);
	VecInt VecFind;
	FindedDist.push_back(VecFind);
}

/*============================================================================
============================== LOCAL FUNCTIONS ===============================
============================================================================*/


/*============================================================================
============================= EXPORTED FUNCTIONS =============================
============================================================================*/

