/*
  RequestDictionary.h

  Request Dictionary Implementation

  Autor: Daniela Slavkova
*/

#ifndef REQUESTDICTIONARY_H
#define REQUESTDICTIONARY_H 

/*--------------------------------------------------------------------------
  Included files to resolve specific definitions in this file
--------------------------------------------------------------------------*/
#include "stdafx.h"
#include <string>
#include <LIST>
#include <VECTOR>
#include "Alphabet.h"
#include "Levenstien.h"

using namespace std;

typedef list<char>	Word;
typedef list<string> GoodSearchResults;	
typedef vector<string> RequestedSequence;
typedef vector<Levenstien*> LevenstienObjs; 
typedef vector<int> VecInt;
 

/*-------------------------------------------------------------------------
  Constant data
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
  Exported data
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
  Class declaration
-------------------------------------------------------------------------*/
// Deserialization of Dictionary
class Dictionary
{
public:
	// Constructor
	Dictionary()
	{
		NewCurrPos = 0;
		InitialState = 0;

		SerializeFile = new char[strlen("TempFile.txt") + 1]; 
		strcpy(SerializeFile, "TempFile.txt");

//		LevenstienObj = NULL;

		FlagDictDeserialization = false;
		FlagExecutionErrors = false;
	}

	// Constructor
	Dictionary(char* SerFile/*, int LevenstienDist, string Request*/)
	{
		NewCurrPos = 0;
		InitialState = 0;
		
		SerializeFile = new char[strlen(SerFile) + 1];  
		strcpy(SerializeFile, SerFile);

//		LevenstienObj = NULL;

		FlagDictDeserialization = false;
		FlagExecutionErrors = false;
	}
	
	// Destructor
	~Dictionary()
	{
		delete SerializeFile;
		if(FlagDictDeserialization == true)
		{
			delete [] Dict;
			delete [] Fin;
		}
	}
	
	void DeserializeArray();
	void TraversingDictionary();
	void RequestDictionary(string Request, int LevenstienDistance);
	void PrintGoodWords(char* ResFile);

	bool FlagExecutionErrors;
private:
	void InitializeDictionary();
	void Traversing(int State);
	void Requesting(int State, int Index,  int PositionString);
	void RequestLevenstien(int InitialSubstringState, int PositionString, int LevenstienDistance);
	void PrintVisited();
	string ToString(int MinFindLevenshtienDist);
	string TranslateWord(string T);
	int RequestAnswer();
	string FindRequest(string Request);
	string OperateAbbreviationInRequest(string Request);
	string RemoveLastIntervals(string Request);
	void NewLevenshtienObject(string Pattern, int LevenshtienDist);

	int NewCurrPos;						// states count; point new possible position in Dict
	int InitialState;					// Initial state in dictionary

	GoodSearchResults SearchedWords;
//	Levenstien *LevenstienObj[];			// Levenstien distance class

	LevenstienObjs LevenstienObj;

	int (*Dict)[CharMapL];				// Dictionary of states and transitions 
	int *Fin;							// Final states indicator array

	Word Visited;					    // Visited letters

	char* SerializeFile;				// Serializating file name

	bool FlagDictDeserialization;		// Marker for deserialization availability 

	vector<VecInt> MarkersDisplay;		// Marker for finded good words in some sequence in ditionary
	vector<VecInt> FindedDist;			// Good distance in sequence
}; 


/*-------------------------------------------------------------------------
  Exported functions
-------------------------------------------------------------------------*/


#endif /* REQUESTDICTIONARY_H */


